@@ -31,6 +31,8 @@
 #include <libglom/privs.h>
 #include <glom/xsl_utils.h>
 #include <glom/python_embed/glom_python.h>
+#include <glom/print_layout/print_layout_utils.h>
+#include <glom/application.h>
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
@@ -642,22 +644,28 @@ void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr
   if(Conversions::value_is_empty(field_value))
     return; //Ignore empty ID fields.
 
+  //Updating doesn't seem necessary. The field details seem to be full already.
+  //Update the field details from the document:
+  ////sharedptr<LayoutItem_Field> unconst_field = sharedptr<LayoutItem_Field>::cast_const(layout_field); //A hack, because layout_field_should_have_navigation() needs to get full field details.
+  //unconst_field->set_full_field_details(
+  //  document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
+      
+  sharedptr<Relationship> field_used_in_relationship_to_one;
+  const bool has_open_button = 
+    DbUtils::layout_field_should_have_navigation(m_table_name, layout_field, get_document(), 
+    field_used_in_relationship_to_one);
+         
   //If it's a simple field that is part of a relationship,
   //identifying a related record.
-  sharedptr<const Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layout_field);
-  if(relationship)
+  if(field_used_in_relationship_to_one)
   {
-    signal_requested_related_details().emit(relationship->get_to_table(), field_value);
+    signal_requested_related_details().emit(field_used_in_relationship_to_one->get_to_table(), field_value);
     return;
   }
 
   //If it is a related field that is a primary key,
   //meaning it identifies a record in another table:
-  sharedptr<const Field> field_info = layout_field->get_full_field_details();
-  const bool field_is_related_primary_key =
-    layout_field->get_has_relationship_name() &&
-    field_info && field_info->get_primary_key();
-  if(field_is_related_primary_key)
+  if(has_open_button)
   {
     signal_requested_related_details().emit(layout_field->get_table_used(m_table_name), field_value);
   }
@@ -900,100 +908,40 @@ sharedptr<Field> Box_Data_Details::get_field_primary_key() const
   return m_field_primary_key;
 }
 
-void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const sharedptr<const LayoutGroup>& group)
-{
-  xmlpp::Element* nodeChildGroup = node_parent->add_child("group");
-  nodeChildGroup->set_attribute("title", group->get_title());
-
-  LayoutGroup::type_list_const_items items = group->get_items();
-  for(LayoutGroup::type_list_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
-  {
-    sharedptr<const LayoutItem> layout_item = *iter;
-
-    sharedptr<const LayoutGroup> pLayoutGroup = sharedptr<const LayoutGroup>::cast_dynamic(layout_item);
-    if(pLayoutGroup)
-      print_layout_group(nodeChildGroup, pLayoutGroup); //recurse
-    else
-    {
-      sharedptr<const LayoutItem_Field> pLayoutField = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
-      if(pLayoutField)
-      {
-        xmlpp::Element* nodeField = nodeChildGroup->add_child("field");
-
-        nodeField->set_attribute("title", pLayoutField->get_title_or_name());
-
-        Gnome::Gda::Value value = m_FlowTable.get_field_value(pLayoutField);
-        const Glib::ustring text_representation = Conversions::get_text_for_gda_value(pLayoutField->get_glom_type(), value,
-          pLayoutField->get_formatting_used().m_numeric_format); //In the current locale.
-
-        nodeField->set_attribute("value", text_representation);
-      }
-      else
-      {
-        sharedptr<const LayoutItem_Portal> pLayoutPortal = sharedptr<const LayoutItem_Portal>::cast_dynamic(layout_item);
-        if(pLayoutPortal)
-        {
-          xmlpp::Element* nodePortal = nodeChildGroup->add_child("related_records");
-
-          //Box_Data_List_Related* pPortalWidget = m_FlowTable.get_portals();
-
-          sharedptr<Relationship> relationship = get_document()->get_relationship(m_table_name, pLayoutPortal->get_relationship_name());
-          if(relationship)
-          {
-            nodePortal->set_attribute("title", relationship->get_title_or_name());
-
-            //TODO:
-
-            //TODO: Only print this if the user has access rights.
-          }
-        }
-      }
-    }
-  }
-
-}
-
 void Box_Data_Details::print_layout()
 {
   const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
+    return;  //TODO: Warn the user.
+   
+  const Document* document = dynamic_cast<const Document*>(get_document());
+  if(!document)
   {
-    //TODO: Warn the user.
+    std::cerr << G_STRFUNC << ": document was null" << std::endl;
+    return;
   }
-  else
+
+  Glib::RefPtr<Gtk::PageSetup> page_setup = Gtk::PageSetup::create(); //TODO: m_canvas.get_page_setup();
+  if(!page_setup)
   {
-    //Create a DOM Document with the XML:
-    xmlpp::DomParser dom_parser;;
-
-    xmlpp::Document* pDocument = dom_parser.get_document();
-    xmlpp::Element* nodeRoot = pDocument->get_root_node();
-    if(!nodeRoot)
-    {
-      //Add it if it isn't there already:
-      nodeRoot = pDocument->create_root_node("details_print");
-    }
-
-    Glib::ustring table_title = get_document()->get_table_title(m_table_name);
-    if(table_title.empty())
-      table_title = m_table_name;
-
-    nodeRoot->set_attribute("table", table_title);
-
-
-    //The groups:
-    xmlpp::Element* nodeParent = nodeRoot;
-
-    const Document::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name, m_layout_platform);
-    for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
-    {
-      sharedptr<const LayoutGroup> layout_group = *iter;
-      print_layout_group(nodeParent, layout_group);
-    }
-
-    GlomXslUtils::transform_and_open(*pDocument, "print_details_to_html.xsl", get_app_window());
+    std::cerr << G_STRFUNC << ": page_setup was null" << std::endl;
+    return;
   }
+
+  //Note that we initially create the page layout without spaces for page 
+  //breaks because those spaces would be empty space on the page after
+  //we have moved items down when expanding:
+  //TODO: Squash that space when expanding custom layouts.
+  sharedptr<PrintLayout> print_layout = 
+    PrintLayoutUtils::create_standard(page_setup, m_table_name, document,
+      false /* do not avoid page margins */);
+  
+  //Show the print preview window:
+  Application* app = Application::get_application();
+  PrintLayoutUtils::do_print_layout(print_layout, m_found_set,
+    false /* not preview */, document, true /* avoid page margins */, app);
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -1025,4 +973,9 @@ void Box_Data_Details::do_new_record()
   on_button_new();
 }
 
+void Box_Data_Details::set_enable_drag_and_drop(bool enabled)
+{
+  m_FlowTable.set_enable_drag_and_drop(enabled);
+}
+
 } //namespace Glom
